import numpy as np
import matplotlib.pyplot as plt
from readSEPfile import read_in_data_from_filepath
from matplotlib.patches import Rectangle

def plot_SDASA_data(data, start_ch=0, clip_val=98, title=None, sampling_rate=50):
    nchannel = data.shape[0]
    end_ch = start_ch+nchannel
    wlen = data.shape[1]*1./sampling_rate
    clipval = np.percentile(np.absolute(data),clip_val)
    plt.imshow(data,aspect='auto',interpolation='nearest',cmap='gray',extent=[0,wlen,end_ch,start_ch],vmin=-clipval,vmax=clipval)
    plt.xlabel('Time [s]')
    plt.ylabel('Channel #')
    if title is not None:
        plt.title(title)
    plt.show()

def plot_SDASA_broadband(DAS_data, bb_data, start_ch=0, clip_val=98, title=None, sampling_rate=50, file_path=None, \
                        want_line=False, sp=[59, 154, 186, 283, 429], col=['r', 'r', 'k', 'b', 'b'], \
                        linestyle=['--','--','-','--','--'], label_segments=True):
    nchannel = DAS_data.shape[0]
    end_ch = start_ch+nchannel
    wlen = DAS_data.shape[1]*1./sampling_rate
    clipval = np.percentile(np.absolute(DAS_data),clip_val)
    t_axis = np.linspace(0, wlen, DAS_data.shape[1])
    f, (a0, a1) = plt.subplots(2,1,gridspec_kw = {'height_ratios':[4, 1]})
    if want_line:
        for icol, ch in enumerate(sp):
            a0.plot(t_axis, [ch]*t_axis.size, col[icol], linestyle=linestyle[icol])
    a0.imshow(DAS_data,aspect='auto',interpolation='nearest',cmap='gray',extent=[0,wlen,end_ch,start_ch],vmin=-clipval,vmax=clipval)
    if label_segments:
        color_ls = ['r']*3+['b']*3 
        x_pos = 0.08
        y_post_ls = [0.92, 0.8, 0.72, 0.63, 0.46, 0.32]
        label_ls = ['r1', 'r2', 'r3', 'b1', 'b2', 'b3']
        for ic, col in enumerate(color_ls):
            f.text(x_pos,y_post_ls[ic], label_ls[ic], fontdict={'color':col, 'weight':'bold'})
        a0.get_yaxis().set_visible(False)
    a0.get_xaxis().set_visible(False)
    #a0.xlabel('Time [s]')
    #a0.ylabel('Channel #')
    x_axis = np.linspace(0, wlen, bb_data.size)
    a1.plot(x_axis, bb_data)
    a1.set_xlim([0,wlen])
    a1.grid()
    f.tight_layout()
    a1.set_xlabel('Time [s]')
    if file_path is not None:
        f.savefig(file_path)

def plot_SDASA_2broadband(DAS_data, bb1_data, bb2_data, start_ch=0, clip_val=98, title=None, \
                        sampling_rate=50, file_path=None, \
                        want_line=False, sp=[59, 154, 186, 283, 429], col=['r', 'r', 'k', 'b', 'b'], \
                        linestyle=['--','--','-','--','--'], label_segments=True, plot_rec=False, \
                        bttm_lft_coor=None, rec_width=None, rec_height=None, want_channel_label=False):
    nchannel = DAS_data.shape[0]
    end_ch = start_ch+nchannel
    wlen = DAS_data.shape[1]*1./sampling_rate
    clipval = np.percentile(np.absolute(DAS_data),clip_val)
    t_axis = np.linspace(0, wlen, DAS_data.shape[1])
    f, (a0, a1, a2) = plt.subplots(3,1,gridspec_kw = {'height_ratios':[8, 1, 1]})
    if want_line:
        for icol, ch in enumerate(sp):
            a0.plot(t_axis, [ch]*t_axis.size, col[icol], linestyle=linestyle[icol])
    a0.imshow(DAS_data,aspect='auto',interpolation='nearest',cmap='gray',extent=[0,wlen,end_ch,start_ch],vmin=-clipval,vmax=clipval)
    if not want_channel_label:
        a0.get_yaxis().set_visible(False)
    if label_segments:
        color_ls = ['r']*3+['b']*3 
        x_pos = 0.08
        y_post_ls = [0.92, 0.8, 0.72, 0.63, 0.46, 0.32]
        label_ls = ['r1', 'r2', 'r3', 'b1', 'b2', 'b3']
        for ic, col in enumerate(color_ls):
            f.text(x_pos,y_post_ls[ic], label_ls[ic], fontdict={'color':col, 'weight':'bold'})
        a0.get_yaxis().set_visible(False)
    a0.get_xaxis().set_visible(False)
    #a0.xlabel('Time [s]')
    #a0.ylabel('Channel #')
    bb1_x_axis = np.linspace(0, wlen, bb1_data.size)
    bb2_x_axis = np.linspace(0, wlen, bb2_data.size)
    a1.plot(bb1_x_axis, bb1_data)
    a1.get_xaxis().set_visible(False)
    a1.tick_params(labelsize=8)
    a2.plot(bb2_x_axis, bb2_data)
    a2.tick_params(labelsize=8)
    a1.set_xlim([0,wlen])
    #a1.grid()
    a2.set_xlim([0,wlen])
    #a2.grid()
    a2.set_xlabel('Time [s]')
    f.tight_layout()
    f.subplots_adjust(hspace=0.02)
    if plot_rec:
        bttm_lft_coor0,bttm_lft_coor1,bttm_lft_coor2 = bttm_lft_coor
        rec_width0,rec_width1,rec_width2 = rec_width
        rec_height0,rec_height1,rec_height2 = rec_height
        rect0 = Rectangle(xy=bttm_lft_coor0, width=rec_width0, height=rec_height0, edgecolor='orange', \
                          linewidth=1.5, fill=False)
        rect1 = Rectangle(xy=bttm_lft_coor1, width=rec_width1, height=rec_height1, edgecolor='orange', \
                          linewidth=1.5, fill=False)
        rect2 = Rectangle(xy=bttm_lft_coor2, width=rec_width2, height=rec_height2, edgecolor='orange', \
                          linewidth=1.5, fill=False)
        a0.add_patch(rect0)
        a1.add_patch(rect1)
        a2.add_patch(rect2)
    if file_path is not None:
        f.savefig(file_path)
