import sys
import os
from reader import readTrace
from obspy import UTCDateTime
import sqlite3
import scipy.fftpack as ft
import regularFileSet as rfs
import obspy
import numpy as np
import datetime as dt
import matplotlib.pyplot as plt
import time 
from multiprocessing import Pool
from scipy.interpolate import interp1d


def plot_numpy_matrix(data, clip_val=90, sampling_rate=50):
    n_ch,n_samp = data.shape
    wlen = n_samp*1./sampling_rate
    clipval = np.percentile(np.absolute(data),clip_val)
    fig,ax = plt.subplots()
    ax.imshow(data,aspect='auto',interpolation='nearest',cmap='gray',extent=[0,wlen,n_ch,1],vmin=-clipval,vmax=clipval)
    plt.xlabel('Time [s]')
    plt.ylabel('Channel #')
    plt.show()

def remove_comm_noisy_channel(data1, data2, percentile=85):
    data1, dead_idx1 = kill_dead_trs(data1)
    data2, dead_idx2 = kill_dead_trs(data2)
    n_ch = data1.shape[0]
    channel_list = np.arange(n_ch)
    dead_idx = np.unique(np.concatenate((dead_idx1,dead_idx2)))
    good_idx = np.setdiff1d(channel_list,dead_idx)
    data1 = data1[good_idx,:]
    data2 = data2[good_idx,:]
    return data1, data2

def kill_dead_trs(data, threshold=3):
    max_vec = np.amax(np.absolute(data), axis=1)
    med_max_vec = np.median(max_vec)
    dead_idx = np.where(max_vec>(threshold*med_max_vec))
    data[dead_idx,:] = np.zeros((data.shape[1]))
    return data, dead_idx[0]

def taper_npy_array(data,taper_len,sample_rate):
    tapered_data = []
    stats = {'sampling_rate':sample_rate}
    for idx in range(0, len(data)):
        x = data[idx,:]
        x_Tr = obspy.core.trace.Trace(data=x,header=stats)
        x_Tr.taper(max_percentage=None, max_length=taper_len)
        x = x_Tr.data
        tapered_data.append(x)
    tapered_data = np.asarray(tapered_data)
    return tapered_data

def coherence(data1, data2, sampling_rate, smooth_dgr=5,taper_len=0.5,freq_log_spacing=0.025):
    if len(data1.shape) == 1:
        data1 = data1.reshape(1,data1.size)
        data2 = data2.reshape(1,data2.size)
    data1 = taper_npy_array(data1,taper_len,sampling_rate)
    data2 = taper_npy_array(data2,taper_len,sampling_rate)
    fft1 = np.fft.fft(data1, axis=1)
    fft2 = np.fft.fft(data2, axis=1)
    n_sample = data1.shape[1]
    n_ch = data1.shape[0]
    freq = np.fft.fftfreq(n_sample, d=1./sampling_rate)
    end_idx = int(round(n_sample/2))
    fft1 = fft1[:,:end_idx]
    fft2 = fft2[:,:end_idx]
    freq_axis = freq[:end_idx]
    f1 = interp1d(freq_axis,fft1,'cubic')
    f2 = interp1d(freq_axis,fft2,'cubic')
    freq_log_st = np.log10(freq_axis[1])
    freq_log_ed = np.log10(freq_axis[-1])
    freq_log_num = int(round((freq_log_ed-freq_log_st)/freq_log_spacing+1))
    freq_axis = np.logspace(freq_log_st, freq_log_ed, freq_log_num)
    fft1 = f1(freq_axis)
    fft2 = f2(freq_axis)
    for ch in range(n_ch):
        fft1[ch,:] = smoothTriangle(fft1[ch,:], smooth_dgr)
        fft2[ch,:] = smoothTriangle(fft2[ch,:], smooth_dgr)
    spec1 = np.absolute(fft1)
    spec2 = np.absolute(fft2)

    cross_Spec = np.conj(fft1)*fft2
    coherent = cross_Spec**2/fft1/fft2
    spec_ratio = np.absolute(spec1)/np.absolute(spec2)
    return spec1, spec2, cross_Spec, coherent, spec_ratio, freq_axis

'''
def spec_ratio(data1, data2, sampling_rate, taper_percentage, smooth_dgr=3):
    n_t = data1.shape[0]
    stats = {'sampling_rate':sampling_rate}
    trace1 = obspy.core.trace.Trace(data=data1, header=stats)
    trace2 = obspy.core.trace.Trace(data=data2, header=stats)

    trace1.taper(max_percentage=taper_percentage, max_length=None, type='hann')
    trace2.taper(max_percentage=taper_percentage, max_length=None, type='hann')

    trace1.data = trace1.data-np.mean(trace1.data)
    trace2.data = trace2.data-np.mean(trace2.data)

    spec1 = np.absolute(np.fft.fft(trace1.data))
    spec2 = np.absolute(np.fft.fft(trace2.data))

    spec1 = smoothTriangle(spec1, smooth_dgr)
    spec2 = smoothTriangle(spec2, smooth_dgr)

    spec_ratio = np.absolute(spec1)/np.absolute(spec2)
    #spec_ratio = spec1/(spec2+args.eps)

    freq = np.fft.fftfreq(n_t, d=1./sampling_rate)
    return freq[:(n_t/2)], spec_ratio[:(n_t/2)], spec1[:(n_t/2)], spec2[:(n_t/2)]
'''
def smoothTriangle(data,degree,dropVals=False):
    """performs moving triangle smoothing with a variable degree."""
    """note that if dropVals is False, output length will be identical
    to input length, but with copies of data at the flanking regions"""
    triangle=np.array(range(degree)+[degree]+range(degree)[::-1])+1
    smoothed=[]
    for i in range(degree,len(data)-degree*2):
        point=data[i:i+len(triangle)]*triangle
        smoothed.append(sum(point)/sum(triangle))
    if dropVals: return smoothed
    smoothed=[smoothed[0]]*(degree+degree/2)+smoothed
    while len(smoothed)<len(data):smoothed.append(smoothed[-1])
    return np.array(smoothed)

def locateDASData(starttime, length, sqliteRFSCursor, channelList, if_YMD=True, if_check_sampling_rate=True):
    parts = ['cbt_processed_','year4','month','day','_','hour24start0','minute','second','.','millisecond','+0000.sgy']
    starttime = UTCDateTime(starttime)
    if starttime >= UTCDateTime(2017):
        starttime = starttime+1 # because of the leap second issue of DAS recording
    endtime = starttime+length
    RFSC = sqliteRFSCursor
    RFSC.execute("SELECT DISTINCT * FROM regularFileSets WHERE NOT (startTime >= ? OR endTime <= ?) order by startTime", (endtime.datetime, starttime.datetime))
    relevantRows = RFSC.fetchall()
    if len(relevantRows) > 0:
        fileSetList = []
        for row in relevantRows:
            thisSetStartTime = row[0]
            thisSetEndTime = row[1]
            secondsPerFile = row[2]
            nFiles = row[3]
            fileSetList.append(rfs.regularFileSet(parts, thisSetStartTime.year, thisSetStartTime.month, thisSetStartTime.day, thisSetStartTime.hour, thisSetStartTime.minute, thisSetStartTime.second,int(0.001*thisSetStartTime.microsecond),secondsPerFile,nFiles))
            fileSetsDisjoint = False
            if len(fileSetList) > 2:
                fileSetsDisjoint = thisSetStartTime > fileSetList[-2].endTime
            if fileSetsDisjoint:
                return False
        eventIncluded = ((fileSetList[0].startTime < starttime) and (fileSetList[-1].endTime > endtime))
        results_to_return = readDASData(fileSetList, starttime, length, channelList)
        if eventIncluded and not isinstance(results_to_return ,bool):
            return results_to_return
        else:
            return False
    return False

def readDASData_single_file(parameters):
        fileList, thisFileSet, fileSetList, dataDir, if_YMD, fileName, channelList, \
                  channelList_int, starttime, endtime, sampling_rate, if_check_sampling_rate = parameters
        thisFileStartTime = UTCDateTime(thisFileSet.getTimeFromFilename(fileName))
        twoDigMonth = str(thisFileStartTime.month)
        if if_YMD:
            if(thisFileStartTime.month < 10):
                twoDigMonth = '0'+twoDigMonth
            twoDigDay = str(thisFileStartTime.day)
            if(thisFileStartTime.day < 10):
                twoDigDay = '0'+twoDigDay
            dataDir = os.path.join(dataDir,str(thisFileStartTime.year), twoDigMonth, twoDigDay)
        data_path = os.path.join(dataDir, fileName)
        print data_path
        if if_check_sampling_rate:
            st = obspy.read(data_path,format='segy',headonly=True)
            sampling_rate_file = st.traces[0].stats.sampling_rate
            if sampling_rate != sampling_rate_file:
                print "input sampling rate ({}) is not equal to \
                        the actual sampling rate ({})...".format(sampling_rate, sampling_rate_file)
                return False
        deltaSec = 1./sampling_rate
        secondsBetweenFiles = thisFileSet.secondsBetweenFiles
        thisFileEndTime = thisFileStartTime + dt.timedelta(seconds=secondsBetweenFiles)
        thisStartTime = thisFileStartTime
        thisEndTime = thisFileEndTime
        if(thisStartTime < starttime):
            thisStartTime = starttime
        if(thisEndTime > endtime):
            thisEndTime = endtime
        startIdxReading = 0
        if starttime > thisFileStartTime:
            secondsAfterStart = (starttime - thisFileStartTime)
            startIdxReading = int(sampling_rate*secondsAfterStart)
        samplesPerFile = int(secondsBetweenFiles*sampling_rate)
        endIdxReading = samplesPerFile
        if endtime < thisFileEndTime:
            secondsAfterStart = (endtime - thisFileStartTime)
            endIdxReading = int(sampling_rate*secondsAfterStart)
        nIdxToRead = endIdxReading-startIdxReading
        acturalStartTime = thisStartTime-1
        data_rate = []
        for ich,channel in enumerate(channelList):
            ch = channelList_int[channel]
            data = readTrace(data_path,samplesPerFile,4,ch,'>',startIdxReading,nIdxToRead)
            if ich == 0:
                data_rate = data
            else:
                data_rate = np.vstack((data_rate, data))
        return data_rate
    

def readDASData(fileSetList, starttime, length, channelList, sampling_rate=50, \
            channelList_File_name="/home/syyuan/project/SDASA/creatEventCatalog/Txt/channelsInOrderAlongFiber.txt",\
            dataDir='/data/biondo/DAS/', if_YMD=True, if_check_sampling_rate=False):
    channelListFile = open(channelList_File_name,'r')
    channelList_int = [int(num) for num in channelListFile.readlines()] # channel list in the interlacing order
    channelListFile.close()
    endtime = starttime+length
    dataRate = [0]*len(channelList)
    DAS_data_results = []
    for ifs, thisFileSet in enumerate(fileSetList):
        fileList = thisFileSet.getFileNamesInRange(starttime.datetime, endtime.datetime)
        parameters_list = [(fileList, thisFileSet, fileSetList, dataDir, if_YMD, fileName, channelList,\
                           channelList_int, starttime, endtime, sampling_rate, if_check_sampling_rate) for fileName in fileList] 
        for ip, parameters in enumerate(parameters_list):
            DAS_data = readDASData_single_file(parameters)
            if isinstance(DAS_data, bool):
                return False
            if ifs==0 and ip ==0:
                DAS_data_results = DAS_data
            else:
                DAS_data_results = np.concatenate((DAS_data_results, DAS_data), axis=1)
    if abs(DAS_data_results.size - length*sampling_rate*len(channelList)) > 0.00001:
        print DAS_data_results.shape, length*sampling_rate*len(channelList)
        return False
    return DAS_data_results

def bandpass(DASData, FreqLow, FreqHi, taperLen=0.5):
    ''' Bandpass '''
    stats = {'sampling_rate':50}
    DASDataTrace = obspy.core.trace.Trace(data=DASData, header=stats)
    DASDataTrace.taper(max_percentage=None, max_length=taperLen)
    DASDataTrace.filter('bandpass', freqmin=FreqLow,freqmax=FreqHi,zerophase=True)
    return DASDataTrace.data

def getDASStrainRate(starttime, wlen, channel_list, bp=False, \
                    flo=0.5, fhi=20, sampling_rate=50, \
                    rfsLog='/home/syyuan/project/SDASA/regulerFileSetDB/logs/regFileSetLog.sqlite'):
    conn = sqlite3.connect(rfsLog, detect_types=sqlite3.PARSE_DECLTYPES)
    c = conn.cursor()
    results = locateDASData(starttime, wlen+2./sampling_rate, c, channel_list)
    if isinstance(results, bool):
        return False
    strain = results
    strain = np.asarray(strain)
    n_ch = len(channel_list)
    n_sam = int(round(strain.size/n_ch))
    strain = strain.reshape((n_ch, n_sam))
    strain_rate = strain[:,1:]-strain[:,:-1]
    for ic, colume in enumerate(strain_rate.T):
        strain_rate[:, ic] = strain_rate[:, ic]-np.median(strain_rate[:,ic])
    return strain_rate


def get_file_dir(root,tmplt_folder, date_time):
    file_dir = os.path.join(root,tmplt_folder,"{:0>4d}".format(date_time.year),\
               "{:0>2d}".format(date_time.month),\
               "{:0>2d}".format(date_time.day))
    return file_dir

def get_time_from_file_name(file_name):
    return UTCDateTime(file_name.split('.')[0])
def get_time_from_file_path(file_path):
    file_name = file_path.split('/')[-1]
    return get_time_from_file_name(file_name)

def name_data_file(date_time):
    file_name = "{:0>4d}{:0>2d}{:0>2d}T{:0>2d}{:0>2d}{:0>2d}.npz".format(date_time.year,date_time.month,\
                date_time.day,date_time.hour,date_time.minute,date_time.second)
    return file_name

def get_file_path_before_time_one_dir(file_dir, date_time):
    file_arr = os.listdir(file_dir)
    file_name_arr = [os.path.join(file_dir,fn) for fn in file_arr \
                    if get_time_from_file_name(fn) <= date_time]
    file_name_arr.sort()
    return file_name_arr
     
    
def get_file_path_after_time_one_dir(file_dir, date_time):
    file_arr = os.listdir(file_dir)
    file_name_arr = [os.path.join(file_dir, file_name) for file_name in file_arr \
                    if get_time_from_file_name(file_name) >= date_time]
    file_name_arr.sort()
    return file_name_arr

def get_file_path_from_time_one_dir(file_dir):
    file_arr = os.listdir(file_dir)
    file_name_arr = [os.path.join(file_dir, file_name) for file_name in file_arr]
    file_name_arr.sort()
    return file_name_arr

def get_file_dir_from_time_range(root, tmplt_folder, start_date_time, end_date_time):
    assert (end_date_time >= start_date_time)
    dir1 = get_file_dir(root, tmplt_folder, start_date_time)
    dir2 = get_file_dir(root, tmplt_folder, end_date_time)
    dirs = []
    if dir1 == dir2:
        return [dir1] 
    day1 = UTCDateTime(start_date_time.year,start_date_time.month,start_date_time.day)
    day2 = UTCDateTime(end_date_time.year,end_date_time.month,end_date_time.day)
    day_range = np.arange(day1,day2,24*3600)
    for day in day_range:
        dirs.append(get_datafile_from_time(root, tmplt_folder,day)[0])
    dirs.append(dir2)
    return dirs

def get_datafile_from_time(root, tmplt_folder, date_time, sampling_rate=50):
    file_dir = get_file_dir(root, tmplt_folder, date_time) 
    file_arr = os.listdir(file_dir)
    file_time_arr = [get_time_from_file_name(file_name) for file_name in file_arr \
                    if get_time_from_file_name(file_name) <= date_time]
    assert (len(file_time_arr)>0), "start time is not included in any results folder!"
    file_time = min(file_time_arr, key=lambda x:abs(x-date_time))
    file_end_time = get_file_end_time(root, tmplt_folder, file_time, sampling_rate)
    assert (file_end_time > date_time), "start time is not included in any results folder!"
    file_name = name_data_file(file_time)
    return file_dir, file_name

     
def get_file_paths_from_time_range(root, tmplt_folder, start_date_time, end_date_time, sampling_rate=50,include_start=True):
    assert (end_date_time>start_date_time)
    dirs = get_file_dir_from_time_range(root, tmplt_folder, start_date_time, end_date_time)
    files = []
    file_start_dir, file_start = get_datafile_from_time(root, tmplt_folder, start_date_time, sampling_rate)
    file_start = os.path.join(file_start_dir, file_start)
    files1 = get_file_path_after_time_one_dir(dirs[0], start_date_time)
    files2 = get_file_path_before_time_one_dir(dirs[-1], end_date_time)
    if len(dirs) == 1:
        files = list(set(files1) & set(files2))
    else:
        for idir, dr in enumerate(dirs):
            if idir == 0:
                files = files+files1
            elif dr == dirs[-1]:
                files = files+files2
            else:
                files = files + get_file_path_from_time_one_dir(dr)
    files2.sort()
    if files == []:
        files = [files2[-1]]
    if include_start and file_start != files[0]:
        files = [file_start]+files 
    files = list(set(files))
    files.sort()
    return files

def get_file_end_time(root, tmplt_folder, file_time, sampling_rate=50):
    file_name = name_data_file(file_time)
    file_dir = get_file_dir(root, tmplt_folder, file_time)
    file_path = os.path.join(file_dir, file_name)
    npz_file = np.load(file_path)
    num_samples = npz_file["arr_0"].shape[1]
    file_end_time = file_time+num_samples*1./sampling_rate
    return file_end_time
    

def check_file_cont(root, tmplt_folder, file1, file2):
    file1_start_time = get_time_from_file_name(file1) 
    file2_start_time = get_time_from_file_name(file2) 
    assert (file1_start_time < file2_start_time), "file2 should be after file1!"
    file1_end_time = get_file_end_time(root, tmplt_folder, file1_start_time)
    return file1_end_time==file2_start_time

def get_xcorr_results_from_single_file(parameters):
    file_idx, file_path = parameters
    print file_path
    npz_file = np.load(file_path)
    xcorr_results = npz_file["arr_0"]
    time_list = npz_file["arr_1"]
    xcorr_results = xcorr_results.reshape((xcorr_results.size,))
    time_list = time_list.reshape((time_list.size,))
    dic = {"file_idx":file_idx, "xcorr_results": xcorr_results, "time_list":time_list}
    return dic

def get_xcorr_results_from_time_range(root, tmplt_folder, start_date_time, end_date_time, sampling_rate=50, nthreads=1):
    assert (end_date_time>start_date_time),"end time should be greater than the start time!"
    file_paths = get_file_paths_from_time_range(root, tmplt_folder, start_date_time, end_date_time, sampling_rate)
    parameters_list = [(file_idx, file_path) for file_idx, file_path in enumerate(file_paths)]
    pool = Pool(nthreads)
    results = pool.map(get_xcorr_results_from_single_file, parameters_list) 
    pool.close()
    pool.join()
    xcorr_results =[]
    time_list = []
    for idx in range(len(results)):
        xcorr_results = np.concatenate((xcorr_results, results[idx]["xcorr_results"]))
        time_list = np.concatenate((time_list, results[idx]["time_list"]))
    start_file_path = file_paths[0]
    start_file_time = get_time_from_file_path(start_file_path)
    time_offset = start_date_time-start_file_time
    start_idx = int(time_offset*sampling_rate)
    duration = end_date_time-start_date_time
    n_sample = int(duration*sampling_rate)
    end_idx = start_idx+n_sample
    return xcorr_results[start_idx:end_idx], time_list[start_idx:end_idx]
    
def get_xcorr_results_from_folder(folder, sampling_rate=50, nthreads=5):
    file_names = os.listdir(folder)
    file_paths = [os.path.join(folder, file_name) for file_name in file_names]
    parameters_list = [(file_idx, file_path) for file_idx, file_path in enumerate(file_paths)]
    pool = Pool(nthreads)
    results = pool.map(get_xcorr_results_from_single_file, parameters_list) 
    pool.close()
    pool.join()
    xcorr_results =[]
    time_list = []
    for idx in range(len(results)):
        xcorr_results = np.concatenate((xcorr_results, results[idx]["xcorr_results"]))
        time_list = np.concatenate((time_list, results[idx]["time_list"]))
    return xcorr_results, time_list 

if __name__ == '__main__':
    root = '/data/biondo/syyuan/laderaXcorr/JRSC/BHE'
    tmplt_folder = 'feltlake_170712_Mag13'
    file_time = UTCDateTime("2017-07-13T00:00:00.000000Z")
    print get_file_end_time(root, tmplt_folder,file_time,40)
